from rest_framework import serializers

from discussion.models import Topic
from discussion.serializers import TopicSerializer

from .models import Activity, Answer

from courses_learning_objects.serializers import LearningObjectSerializer as ActivitySerializer, LearningObjectImageSerializer as ActivityImageSerializer


class AnswerSerializer(serializers.ModelSerializer):
    correct = serializers.ReadOnlyField(source='is_correct')
    given = serializers.JSONField()
    topic = serializers.SerializerMethodField()

    class Meta:
        model = Answer
        allow_add_remove = True
        fields = ('id', 'activity', 'correct', 'timestamp', 'given', 'topic')

    def get_topic(self, obj):
        try:
            topic = Topic.objects.get(id=obj.given.get('topic', None))
        except (Topic.DoesNotExist, AttributeError) as e:
            return

        return TopicSerializer(instance=topic, **{'context': self.context}).data
